/* ***************************************************************************
 * DASMx64 a frontend to DASMx disassembler for treating Commodore PRG files
 *
 * DASMx is a freeware disassembler for a range of common microprocessors 
 * It supports code emulation and automatic label generation
 * DASM is copyrighted software by Pete Clare, Version 1.40 is allowed to be
 * freely distributed
 *
 * The DASMx64 frontend is free software under the Unlicense
 * written by Wil in February 2021
 *
 * DASMx64 analyses a PRG file, looks for a an eventual SYS stub and feeds
 * DASMx a file without the first two address bytes.
 * Start address and code entry point are given as parameters instead.
 * If no SYS is found, the code entry point is te start address
 * DASMx is called twice to produce an assembler file and a detailed list file.
 * *************************************************************************** */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

char callstr[256];
char tmpfilename[256];
char *progname;

void error_exit(char *s) {
	fprintf(stderr,"%s\n",s);
	exit(1);
}

void usage(char *s) {
	fprintf(stderr,"%s\n",s);
	fprintf(stderr,"Usage: %s cbmfilename\n",progname);
	exit(1);
}

void prepare_file(char *infile,char *outfile,int *startaddr,int *entryaddr) {
  FILE *fpin,*fpout;
  int sysaddr,sysdetect=0;
  
  if ((fpin = fopen( infile, "rb"))==NULL) error_exit("Error opening input file!");
  if ((fpout= fopen(outfile, "wb"))==NULL) error_exit("Error opening output file!");  

  *startaddr = fgetc(fpin) + 256 * fgetc(fpin);
  *entryaddr = *startaddr; //the default value for the start address is the file start
  
  while(1) {
    int c=fgetc(fpin);
    if (feof(fpin)) break;  
    fputc(c,fpout);
    
    if (c==0x9E) {
      sysdetect=1;
	  sysaddr=0;	
	} else if (sysdetect) {
      	if ((c>='0')&&(c<='9'))
      	  sysaddr = sysaddr*10 + c-'0';
      	else if (c==' ') {
      		continue;
		} else {
		  if (sysaddr>=*startaddr) {
		  	*entryaddr=sysaddr;
		  }
		  sysdetect=0;	
		}
	}
  }

  fclose(fpin);	
  fclose(fpout);
}

int main(int argc, char **argv) {
	int status;
	int startaddr,entryaddr;

	progname=argv[0];

	if (argc==1) usage("No input file given!");
	if (argc>=3) usage("Too many input files given (probably due to a space in filename)!");
	
	if ((!strcmp(argv[1]+strlen(argv[1])-4,".prg"))||(!strcmp(argv[1]+strlen(argv[1])-4,".PRG"))) {
		strncpy(tmpfilename,argv[1],strlen(argv[1])-4);
		strcpy(tmpfilename+strlen(tmpfilename),".tmp");
	} else {
	  sprintf(tmpfilename,"%s.tmp",argv[1]);
	} 
    prepare_file(argv[1],tmpfilename,&startaddr,&entryaddr);


	printf("DASMx.exe -a -c6502 -o%i -e%i -t %s\n",startaddr,entryaddr,tmpfilename);
	sprintf(callstr,"DASMx.exe -a -c6502 -o%i -e%i -t %s",startaddr,entryaddr,tmpfilename);
	status = system(callstr);
	if (status) error_exit("DASMx terminated due to an error!");
	sprintf(callstr,"DASMx.exe -w -c6502 -o%i -e%i -t %s",startaddr,entryaddr,tmpfilename);
	status = system(callstr);
	if (status) error_exit("DASMx terminated due to an error when generating the list file!");
	
	remove (tmpfilename);
	
	printf("\n\nFile %s was disassembled successfully, see .asm and .lst file!\n",argv[1]);
	return 0;
}
